package com.sun.org.apache.xalan.internal.xsltc.dom;

import java.util.Vector;
import com.sun.org.apache.xalan.internal.xsltc.Translet;
import com.sun.org.apache.xml.internal.dtm.DTMAxisIterator;
import com.sun.org.apache.xalan.internal.xsltc.DOM;

public abstract class NodeCounter
{
    public static final int END = -1;
    protected int _node;
    protected int _nodeType;
    protected double _value;
    public final DOM _document;
    public final DTMAxisIterator _iterator;
    public final Translet _translet;
    protected String _format;
    protected String _lang;
    protected String _letterValue;
    protected String _groupSep;
    protected int _groupSize;
    private boolean _separFirst;
    private boolean _separLast;
    private Vector _separToks;
    private Vector _formatToks;
    private int _nSepars;
    private int _nFormats;
    private static final String[] Thousands;
    private static final String[] Hundreds;
    private static final String[] Tens;
    private static final String[] Ones;
    private StringBuilder _tempBuffer;
    protected boolean _hasFrom;
    
    protected NodeCounter(Translet translet, DOM document, DTMAxisIterator iterator) {
        this._node = -1;
        this._nodeType = -1;
        this._value = -2.147483648E9;
        this._separFirst = true;
        this._separLast = false;
        this._separToks = new Vector();
        this._formatToks = new Vector();
        this._nSepars = 0;
        this._nFormats = 0;
        this._tempBuffer = new StringBuilder();
        this._translet = translet;
        this._document = document;
        this._iterator = iterator;
    }
    
    protected NodeCounter(Translet translet, DOM document, DTMAxisIterator iterator, boolean hasFrom) {
        this._node = -1;
        this._nodeType = -1;
        this._value = -2.147483648E9;
        this._separFirst = true;
        this._separLast = false;
        this._separToks = new Vector();
        this._formatToks = new Vector();
        this._nSepars = 0;
        this._nFormats = 0;
        this._tempBuffer = new StringBuilder();
        this._translet = translet;
        this._document = document;
        this._iterator = iterator;
        this._hasFrom = hasFrom;
    }
    
    public abstract NodeCounter setStartNode(int p0);
    
    public NodeCounter setValue(double value) {
        this._value = value;
        return this;
    }
    
    protected void setFormatting(String format, String lang, String letterValue, String groupSep, String groupSize) {
        this._lang = lang;
        this._groupSep = groupSep;
        this._letterValue = letterValue;
        this._groupSize = this.parseStringToAnInt(groupSize);
        this.setTokens(format);
    }
    
    private int parseStringToAnInt(String s) {
        if (s == null) {
            return 0;
        }
        int result = 0;
        boolean negative = false;
        int radix = 10;
        int i = 0;
        int max = s.length();
        if (max > 0) {
            int limit;
            if (s.charAt(0) == '-') {
                negative = true;
                limit = Integer.MIN_VALUE;
                ++i;
            }
            else {
                limit = -2147483647;
            }
            int multmin = limit / radix;
            if (i < max) {
                int digit = Character.digit(s.charAt(i++), radix);
                if (digit < 0) {
                    return 0;
                }
                result = -digit;
            }
            while (i < max) {
                int digit = Character.digit(s.charAt(i++), radix);
                if (digit < 0) {
                    return 0;
                }
                if (result < multmin) {
                    return 0;
                }
                result *= radix;
                if (result < limit + digit) {
                    return 0;
                }
                result -= digit;
            }
            if (negative) {
                if (i > 1) {
                    return result;
                }
                return 0;
            }
            else {
                return -result;
            }
        }
        else {
            return 0;
        }
    }
    
    private final void setTokens(String format) {
        if (this._format != null && format.equals(this._format)) {
            return;
        }
        this._format = format;
        int length = this._format.length();
        boolean isFirst = true;
        this._separFirst = true;
        this._separLast = false;
        this._nSepars = 0;
        this._nFormats = 0;
        this._separToks.clear();
        this._formatToks.clear();
        int j = 0;
        int i = 0;
        while (i < length) {
            char c = format.charAt(i);
            j = i;
            while (Character.isLetterOrDigit(c) && ++i != length) {
                c = format.charAt(i);
            }
            if (i > j) {
                if (isFirst) {
                    this._separToks.addElement(".");
                    boolean separFirst = false;
                    this._separFirst = separFirst;
                    isFirst = separFirst;
                }
                this._formatToks.addElement(format.substring(j, i));
            }
            if (i == length) {
                break;
            }
            c = format.charAt(i);
            j = i;
            while (!Character.isLetterOrDigit(c) && ++i != length) {
                c = format.charAt(i);
                isFirst = false;
            }
            if (i > j) {
                this._separToks.addElement(format.substring(j, i));
            }
            else {
                continue;
            }
        }
        this._nSepars = this._separToks.size();
        this._nFormats = this._formatToks.size();
        if (this._nSepars > this._nFormats) {
            this._separLast = true;
        }
        if (this._separFirst) {
            --this._nSepars;
        }
        if (this._separLast) {
            --this._nSepars;
        }
        if (this._nSepars == 0) {
            this._separToks.insertElementAt(".", 1);
            ++this._nSepars;
        }
        if (this._separFirst) {
            ++this._nSepars;
        }
        return;
    }
    
    public NodeCounter setDefaultFormatting() {
        this.setFormatting("1", "en", "alphabetic", null, null);
        return this;
    }
    
    public abstract String getCounter();
    
    public String getCounter(String format, String lang, String letterValue, String groupSep, String groupSize) {
        this.setFormatting(format, lang, letterValue, groupSep, groupSize);
        return this.getCounter();
    }
    
    public boolean matchesCount(int node) {
        return this._nodeType == this._document.getExpandedTypeID(node);
    }
    
    public boolean matchesFrom(int node) {
        return false;
    }
    
    protected String formatNumbers(int value) {
        return this.formatNumbers(new int[] { value });
    }
    
    protected String formatNumbers(int[] values) {
        int nValues = values.length;
        boolean isEmpty = true;
        for (int i = 0; i < nValues; ++i) {
            if (values[i] != Integer.MIN_VALUE) {
                isEmpty = false;
            }
        }
        if (isEmpty) {
            return "";
        }
        boolean isFirst = true;
        int t = 0;
        int n = 0;
        int s = 1;
        this._tempBuffer.setLength(0);
        StringBuilder buffer = this._tempBuffer;
        if (this._separFirst) {
            buffer.append((String)this._separToks.elementAt(0));
        }
        while (n < nValues) {
            int value = values[n];
            if (value != Integer.MIN_VALUE) {
                if (!isFirst) {
                    buffer.append((String)this._separToks.elementAt(s++));
                }
                this.formatValue(value, (String)this._formatToks.elementAt(t++), buffer);
                if (t == this._nFormats) {
                    --t;
                }
                if (s >= this._nSepars) {
                    --s;
                }
                isFirst = false;
            }
            ++n;
        }
        if (this._separLast) {
            buffer.append((String)this._separToks.lastElement());
        }
        return buffer.toString();
    }
    
    private void formatValue(int value, String format, StringBuilder buffer) {
        char c = format.charAt(0);
        if (Character.isDigit(c)) {
            char zero = (char)(c - Character.getNumericValue(c));
            StringBuilder temp = buffer;
            if (this._groupSize > 0) {
                temp = new StringBuilder();
            }
            String s = "";
            for (int n = value; n > 0; n /= 10) {
                s = (char)(zero + n % 10) + s;
            }
            for (int i = 0; i < format.length() - s.length(); ++i) {
                temp.append(zero);
            }
            temp.append(s);
            if (this._groupSize > 0) {
                for (int i = 0; i < temp.length(); ++i) {
                    if (i != 0 && (temp.length() - i) % this._groupSize == 0) {
                        buffer.append(this._groupSep);
                    }
                    buffer.append(temp.charAt(i));
                }
            }
        }
        else if (c == 'i' && !this._letterValue.equals("alphabetic")) {
            buffer.append(this.romanValue(value));
        }
        else if (c == 'I' && !this._letterValue.equals("alphabetic")) {
            buffer.append(this.romanValue(value).toUpperCase());
        }
        else {
            int min = c;
            int max;
            if ((max = c) >= 945 && c <= 'ω') {
                max = 969;
            }
            else {
                while (Character.isLetterOrDigit((char)(max + 1))) {
                    ++max;
                }
            }
            buffer.append(this.alphaValue(value, min, max));
        }
    }
    
    private String alphaValue(int value, int min, int max) {
        if (value <= 0) {
            return "" + value;
        }
        int range = max - min + 1;
        char last = (char)((value - 1) % range + min);
        if (value > range) {
            return this.alphaValue((value - 1) / range, min, max) + last;
        }
        return "" + last;
    }
    
    private String romanValue(int n) {
        if (n <= 0 || n > 4000) {
            return "" + n;
        }
        return NodeCounter.Thousands[n / 1000] + NodeCounter.Hundreds[n / 100 % 10] + NodeCounter.Tens[n / 10 % 10] + NodeCounter.Ones[n % 10];
    }
    
    static {
        Thousands = new String[] { "", "m", "mm", "mmm" };
        Hundreds = new String[] { "", "c", "cc", "ccc", "cd", "d", "dc", "dcc", "dccc", "cm" };
        Tens = new String[] { "", "x", "xx", "xxx", "xl", "l", "lx", "lxx", "lxxx", "xc" };
        Ones = new String[] { "", "i", "ii", "iii", "iv", "v", "vi", "vii", "viii", "ix" };
    }
}
